// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_H_
#define SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/media_session/public/mojom/media_session.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"


namespace media_session {
namespace mojom {
class MediaSessionInfoDataView;

class MediaSessionDebugInfoDataView;



}  // namespace mojom
}  // namespace media_session

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::media_session::mojom::MediaSessionInfoDataView> {
  using Data = ::media_session::mojom::internal::MediaSessionInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::media_session::mojom::MediaSessionDebugInfoDataView> {
  using Data = ::media_session::mojom::internal::MediaSessionDebugInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace media_session {
namespace mojom {

enum class MediaSessionInfo_SessionState : int32_t {
  kActive,
  kDucking,
  kSuspended,
  kInactive,
  kMinValue = 0,
  kMaxValue = 3,
};

inline std::ostream& operator<<(std::ostream& os, MediaSessionInfo_SessionState value) {
  switch(value) {
    case MediaSessionInfo_SessionState::kActive:
      return os << "MediaSessionInfo_SessionState::kActive";
    case MediaSessionInfo_SessionState::kDucking:
      return os << "MediaSessionInfo_SessionState::kDucking";
    case MediaSessionInfo_SessionState::kSuspended:
      return os << "MediaSessionInfo_SessionState::kSuspended";
    case MediaSessionInfo_SessionState::kInactive:
      return os << "MediaSessionInfo_SessionState::kInactive";
    default:
      return os << "Unknown MediaSessionInfo_SessionState value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(MediaSessionInfo_SessionState value) {
  return internal::MediaSessionInfo_SessionState_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class MediaSession_SuspendType : int32_t {
  kSystem,
  kUI,
  kContent,
  kMinValue = 0,
  kMaxValue = 2,
};

inline std::ostream& operator<<(std::ostream& os, MediaSession_SuspendType value) {
  switch(value) {
    case MediaSession_SuspendType::kSystem:
      return os << "MediaSession_SuspendType::kSystem";
    case MediaSession_SuspendType::kUI:
      return os << "MediaSession_SuspendType::kUI";
    case MediaSession_SuspendType::kContent:
      return os << "MediaSession_SuspendType::kContent";
    default:
      return os << "Unknown MediaSession_SuspendType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(MediaSession_SuspendType value) {
  return internal::MediaSession_SuspendType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class MediaSessionObserverInterfaceBase {};

using MediaSessionObserverPtrDataView =
    mojo::InterfacePtrDataView<MediaSessionObserverInterfaceBase>;
using MediaSessionObserverRequestDataView =
    mojo::InterfaceRequestDataView<MediaSessionObserverInterfaceBase>;
using MediaSessionObserverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<MediaSessionObserverInterfaceBase>;
using MediaSessionObserverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<MediaSessionObserverInterfaceBase>;
class MediaSessionInterfaceBase {};

using MediaSessionPtrDataView =
    mojo::InterfacePtrDataView<MediaSessionInterfaceBase>;
using MediaSessionRequestDataView =
    mojo::InterfaceRequestDataView<MediaSessionInterfaceBase>;
using MediaSessionAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<MediaSessionInterfaceBase>;
using MediaSessionAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<MediaSessionInterfaceBase>;
class MediaSessionInfoDataView {
 public:
  MediaSessionInfoDataView() {}

  MediaSessionInfoDataView(
      internal::MediaSessionInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) const {
    auto data_value = data_->state;
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfo_SessionState>(
        data_value, output);
  }

  MediaSessionInfo_SessionState state() const {
    return static_cast<MediaSessionInfo_SessionState>(data_->state);
  }
  bool force_duck() const {
    return data_->force_duck;
  }
 private:
  internal::MediaSessionInfo_Data* data_ = nullptr;
};

class MediaSessionDebugInfoDataView {
 public:
  MediaSessionDebugInfoDataView() {}

  MediaSessionDebugInfoDataView(
      internal::MediaSessionDebugInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetOwnerDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOwner(UserType* output) {
    auto* pointer = data_->owner.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetStateDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadState(UserType* output) {
    auto* pointer = data_->state.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::MediaSessionDebugInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaSessionObserver_MediaSessionInfoChanged_ParamsDataView {
 public:
  MediaSessionObserver_MediaSessionInfoChanged_ParamsDataView() {}

  MediaSessionObserver_MediaSessionInfoChanged_ParamsDataView(
      internal::MediaSessionObserver_MediaSessionInfoChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInfoDataView(
      MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInfo(UserType* output) {
    auto* pointer = data_->info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::MediaSessionObserver_MediaSessionInfoChanged_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaSession_GetMediaSessionInfo_ParamsDataView {
 public:
  MediaSession_GetMediaSessionInfo_ParamsDataView() {}

  MediaSession_GetMediaSessionInfo_ParamsDataView(
      internal::MediaSession_GetMediaSessionInfo_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaSession_GetMediaSessionInfo_Params_Data* data_ = nullptr;
};

class MediaSession_GetMediaSessionInfo_ResponseParamsDataView {
 public:
  MediaSession_GetMediaSessionInfo_ResponseParamsDataView() {}

  MediaSession_GetMediaSessionInfo_ResponseParamsDataView(
      internal::MediaSession_GetMediaSessionInfo_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInfoDataView(
      MediaSessionInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInfo(UserType* output) {
    auto* pointer = data_->info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::MediaSession_GetMediaSessionInfo_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaSession_GetDebugInfo_ParamsDataView {
 public:
  MediaSession_GetDebugInfo_ParamsDataView() {}

  MediaSession_GetDebugInfo_ParamsDataView(
      internal::MediaSession_GetDebugInfo_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaSession_GetDebugInfo_Params_Data* data_ = nullptr;
};

class MediaSession_GetDebugInfo_ResponseParamsDataView {
 public:
  MediaSession_GetDebugInfo_ResponseParamsDataView() {}

  MediaSession_GetDebugInfo_ResponseParamsDataView(
      internal::MediaSession_GetDebugInfo_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInfoDataView(
      MediaSessionDebugInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInfo(UserType* output) {
    auto* pointer = data_->info.Get();
    return mojo::internal::Deserialize<::media_session::mojom::MediaSessionDebugInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::MediaSession_GetDebugInfo_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaSession_StartDucking_ParamsDataView {
 public:
  MediaSession_StartDucking_ParamsDataView() {}

  MediaSession_StartDucking_ParamsDataView(
      internal::MediaSession_StartDucking_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaSession_StartDucking_Params_Data* data_ = nullptr;
};

class MediaSession_StopDucking_ParamsDataView {
 public:
  MediaSession_StopDucking_ParamsDataView() {}

  MediaSession_StopDucking_ParamsDataView(
      internal::MediaSession_StopDucking_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaSession_StopDucking_Params_Data* data_ = nullptr;
};

class MediaSession_Suspend_ParamsDataView {
 public:
  MediaSession_Suspend_ParamsDataView() {}

  MediaSession_Suspend_ParamsDataView(
      internal::MediaSession_Suspend_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSuspendType(UserType* output) const {
    auto data_value = data_->suspend_type;
    return mojo::internal::Deserialize<::media_session::mojom::MediaSession_SuspendType>(
        data_value, output);
  }

  MediaSession_SuspendType suspend_type() const {
    return static_cast<MediaSession_SuspendType>(data_->suspend_type);
  }
 private:
  internal::MediaSession_Suspend_Params_Data* data_ = nullptr;
};

class MediaSession_Resume_ParamsDataView {
 public:
  MediaSession_Resume_ParamsDataView() {}

  MediaSession_Resume_ParamsDataView(
      internal::MediaSession_Resume_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSuspendType(UserType* output) const {
    auto data_value = data_->suspend_type;
    return mojo::internal::Deserialize<::media_session::mojom::MediaSession_SuspendType>(
        data_value, output);
  }

  MediaSession_SuspendType suspend_type() const {
    return static_cast<MediaSession_SuspendType>(data_->suspend_type);
  }
 private:
  internal::MediaSession_Resume_Params_Data* data_ = nullptr;
};

class MediaSession_AddObserver_ParamsDataView {
 public:
  MediaSession_AddObserver_ParamsDataView() {}

  MediaSession_AddObserver_ParamsDataView(
      internal::MediaSession_AddObserver_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeObserver() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::media_session::mojom::MediaSessionObserverPtrDataView>(
            &data_->observer, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::MediaSession_AddObserver_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace media_session

namespace std {

template <>
struct hash<::media_session::mojom::MediaSessionInfo_SessionState>
    : public mojo::internal::EnumHashImpl<::media_session::mojom::MediaSessionInfo_SessionState> {};

template <>
struct hash<::media_session::mojom::MediaSession_SuspendType>
    : public mojo::internal::EnumHashImpl<::media_session::mojom::MediaSession_SuspendType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::media_session::mojom::MediaSessionInfo_SessionState, ::media_session::mojom::MediaSessionInfo_SessionState> {
  static ::media_session::mojom::MediaSessionInfo_SessionState ToMojom(::media_session::mojom::MediaSessionInfo_SessionState input) { return input; }
  static bool FromMojom(::media_session::mojom::MediaSessionInfo_SessionState input, ::media_session::mojom::MediaSessionInfo_SessionState* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::MediaSessionInfo_SessionState, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media_session::mojom::MediaSessionInfo_SessionState, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media_session::mojom::MediaSessionInfo_SessionState>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::media_session::mojom::MediaSession_SuspendType, ::media_session::mojom::MediaSession_SuspendType> {
  static ::media_session::mojom::MediaSession_SuspendType ToMojom(::media_session::mojom::MediaSession_SuspendType input) { return input; }
  static bool FromMojom(::media_session::mojom::MediaSession_SuspendType input, ::media_session::mojom::MediaSession_SuspendType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::MediaSession_SuspendType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::media_session::mojom::MediaSession_SuspendType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::media_session::mojom::MediaSession_SuspendType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::MediaSessionInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::media_session::mojom::MediaSessionInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::media_session::mojom::internal::MediaSessionInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    mojo::internal::Serialize<::media_session::mojom::MediaSessionInfo_SessionState>(
        CallWithContext(Traits::state, input, custom_context), &(*output)->state);
    (*output)->force_duck = CallWithContext(Traits::force_duck, input, custom_context);
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::media_session::mojom::internal::MediaSessionInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::media_session::mojom::MediaSessionInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::media_session::mojom::MediaSessionDebugInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::media_session::mojom::MediaSessionDebugInfoDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::media_session::mojom::internal::MediaSessionDebugInfo_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    (*output).Allocate(buffer);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype((*output)->name)::BaseType::BufferWriter
        name_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_writer, context);
    (*output)->name.Set(
        name_writer.is_null() ? nullptr : name_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null name in MediaSessionDebugInfo struct");
    decltype(CallWithContext(Traits::owner, input, custom_context)) in_owner = CallWithContext(Traits::owner, input, custom_context);
    typename decltype((*output)->owner)::BaseType::BufferWriter
        owner_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_owner, buffer, &owner_writer, context);
    (*output)->owner.Set(
        owner_writer.is_null() ? nullptr : owner_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->owner.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null owner in MediaSessionDebugInfo struct");
    decltype(CallWithContext(Traits::state, input, custom_context)) in_state = CallWithContext(Traits::state, input, custom_context);
    typename decltype((*output)->state)::BaseType::BufferWriter
        state_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_state, buffer, &state_writer, context);
    (*output)->state.Set(
        state_writer.is_null() ? nullptr : state_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->state.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null state in MediaSessionDebugInfo struct");
    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::media_session::mojom::internal::MediaSessionDebugInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::media_session::mojom::MediaSessionDebugInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace media_session {
namespace mojom {



inline void MediaSessionDebugInfoDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void MediaSessionDebugInfoDataView::GetOwnerDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->owner.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void MediaSessionDebugInfoDataView::GetStateDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->state.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void MediaSessionObserver_MediaSessionInfoChanged_ParamsDataView::GetInfoDataView(
    MediaSessionInfoDataView* output) {
  auto pointer = data_->info.Get();
  *output = MediaSessionInfoDataView(pointer, context_);
}




inline void MediaSession_GetMediaSessionInfo_ResponseParamsDataView::GetInfoDataView(
    MediaSessionInfoDataView* output) {
  auto pointer = data_->info.Get();
  *output = MediaSessionInfoDataView(pointer, context_);
}




inline void MediaSession_GetDebugInfo_ResponseParamsDataView::GetInfoDataView(
    MediaSessionDebugInfoDataView* output) {
  auto pointer = data_->info.Get();
  *output = MediaSessionDebugInfoDataView(pointer, context_);
}













}  // namespace mojom
}  // namespace media_session

#endif  // SERVICES_MEDIA_SESSION_PUBLIC_MOJOM_MEDIA_SESSION_MOJOM_SHARED_H_
